--[[-----------------------------------------------------------------------------
LootButton Widget
Displays an item icon with name, loot source name, and time remaining
-------------------------------------------------------------------------------]]
local Type, Version = "LootButton", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnEnter(frame)
	frame.obj:Fire("OnEnter")
end

local function Control_OnLeave(frame)
	frame.obj:Fire("OnLeave")
end

local function Button_OnClick(frame, button)
	frame.obj:Fire("OnClick", button)
	AceGUI:ClearFocus()
end

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]

local function Layout(self)
	if self.resizing then return end
	local frame = self.frame
	local width = frame.width or frame:GetWidth() or 0
	local image = self.image
	local itemNameLabel = self.itemNameLabel
	local lootSourceLabel = self.lootSourceLabel
	local timeRemainingLabel = self.timeRemainingLabel
	local height = image.height or image:GetHeight() or 0

	itemNameLabel:ClearAllPoints()
	lootSourceLabel:ClearAllPoints()
	timeRemainingLabel:ClearAllPoints()
	image:ClearAllPoints()

	local imagewidth = image:GetWidth()
	local imageheight = image:GetHeight()
	image:SetPoint("TOPLEFT", frame, "TOPLEFT", 7, -7)

	itemNameLabel:SetPoint("TOPLEFT", image, "TOPRIGHT", 3, 0)
	itemNameLabel:SetWidth(width - imagewidth - 14)
	itemNameLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight * 0.3)

	lootSourceLabel:SetPoint("TOPLEFT", image, "BOTTOMLEFT")
	lootSourceLabel:SetWidth(width - 14)
	lootSourceLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 4)

	timeRemainingLabel:SetPoint("TOPLEFT", lootSourceLabel, "BOTTOMLEFT")
	timeRemainingLabel:SetWidth(width - 14)
	itemNameLabel:SetNonSpaceWrap(false)
	timeRemainingLabel:SetFont("Fonts\\FRIZQT__.TTF", imageheight / 4)

	self.resizing = true
	frame:SetHeight(imageheight + lootSourceLabel:GetHeight() + timeRemainingLabel:GetHeight() + 14)
	frame.height = imageheight + lootSourceLabel:GetHeight() + timeRemainingLabel:GetHeight() + 14
	self.resizing = nil
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		-- set the flag to stop constant size updates
		self.resizing = true
		-- height is set dynamically by the text and image size
		self:SetWidth(164)
		self:SetNameText()
		self:SetSourceText()
		self:SetTimeText()
		self:SetImage(nil)
		self:SetImageSize(37, 38)
		self:SetColor()
		self:SetDisabled(false)
		
		-- reset the flag
		self.resizing = nil
		-- run the update explicitly
		Layout(self)
	end,

	["OnWidthSet"] = function(self, width)
		Layout(self)
	end,

	["SetNameText"] = function(self, text)
		self.itemNameLabel:SetText(text)
		Layout(self)
	end,

	["SetSourceText"] = function(self, text)
		self.lootSourceLabel:SetText(text)
		Layout(self)
	end,

	["SetTimeText"] = function(self, text)
		self.timeRemainingLabel:SetText(text)
		Layout(self)
	end,

	["SetColor"] = function(self, r, g, b)
		if not (r and g and b) then
			r, g, b = 1, 1, 1
		end
		self.itemNameLabel:SetVertexColor(r, g, b)
		self:SetUserData("itemNameLabelColor", {r = r, g = g, b = b})
	end,

	["SetImage"] = function(self, path, ...)
		local image = self.image
		image:SetTexture(path)
		
		if image:GetTexture() then
			local n = select("#", ...)
			if n == 4 or n == 8 then
				image:SetTexCoord(...)
			else
				image:SetTexCoord(0, 1, 0, 1)
			end
		end
		Layout(self)
	end,

	["SetImageSize"] = function(self, width, height)
		self.image:SetWidth(width)
		self.image:SetHeight(height)
		Layout(self)
	end,
	
	["SetDisabled"] = function(self, disabled)
		self.disabled = disabled
		if disabled then
			--self.frame:Disable()
			self.itemNameLabel:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			self.lootSourceLabel:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			self.image:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			self.highlight:SetVertexColor(0.5, 0.5, 0.5, 0.5)
		else
			self.frame:Enable()
			local color = self:GetUserData("itemNameLabelColor")
			self.itemNameLabel:SetVertexColor(color.r, color.g, color.b, 1)
			self.lootSourceLabel:SetVertexColor(FACTION_BAR_COLORS[2].r, FACTION_BAR_COLORS[2].b, FACTION_BAR_COLORS[2].g, 1)
			self.image:SetVertexColor(1, 1, 1, 1)
			self.highlight:SetVertexColor(1, 1, 1, 1)
		end
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local PaneBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 5, right = 3, top = 3, bottom = 5 }
}

local function Constructor()

	local frame = CreateFrame("Button", nil, border)
	frame:Hide()
	frame:SetBackdrop(PaneBackdrop)
	frame:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	frame:SetBackdropBorderColor(0.4, 0.4, 0.4)
	
	frame:EnableMouse(true)
	frame:SetScript("OnEnter", Control_OnEnter)
	frame:SetScript("OnLeave", Control_OnLeave)
	frame:SetScript("OnClick", Button_OnClick)
	
	local itemNameLabel = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	itemNameLabel:SetJustifyH("LEFT")
	itemNameLabel:SetJustifyV("TOP")
	itemNameLabel:SetWordWrap(true)

	local lootSourceLabel = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	lootSourceLabel:SetJustifyH("LEFT")
	lootSourceLabel:SetJustifyV("TOP")
	lootSourceLabel:SetVertexColor(FACTION_BAR_COLORS[2].r, FACTION_BAR_COLORS[2].b, FACTION_BAR_COLORS[2].g)
	lootSourceLabel:SetWordWrap(false)
	
	local timeRemainingLabel = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	timeRemainingLabel:SetJustifyH("LEFT")
	timeRemainingLabel:SetJustifyV("TOP")
	timeRemainingLabel:SetVertexColor(BATTLENET_FONT_COLOR.r, BATTLENET_FONT_COLOR.b, BATTLENET_FONT_COLOR.g)
	timeRemainingLabel:SetWordWrap(false)
	
	local image = frame:CreateTexture(nil, nil)

	local highlight = frame:CreateTexture(nil, "HIGHLIGHT")
	highlight:SetAllPoints(frame)
	highlight:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
	--highlight:SetTexCoord(0, 0, 0, 0)
	highlight:SetBlendMode("ADD")

	-- create widget
	local widget = {
		itemNameLabel = itemNameLabel,
		lootSourceLabel = lootSourceLabel,
		timeRemainingLabel = timeRemainingLabel,
		highlight = highlight,
		image = image,
		frame = frame,
		type  = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
